#!/bin/bash

# Thanks Jan Hoffmann from the Ubuntu GNOME Shell Remix for this script
# Download the original at http://sourceforge.net/projects/ubuntu-gs-remix/files/live-script/
# Customized by Jeremy Bicha for the Ubuntu GNOME Remix
# See also https://help.ubuntu.com/community/LiveCDCustomizationFromScratch

usage() {
	echo "Usage: `basename $0` cmd arch [src]"
	echo ' - cmd:  either customize to create a live image or clean to delete the temporary files'
	echo ' - arch: either i386 or amd64'
	echo ' - src:  Ubuntu 12.10 desktop live CD image that is going to be used as base (only for cmd=customize)'
	echo 'Notes:'
	echo ' - The working directory has to be the same the script is in.'
	echo ' - Temporary data will be created in the current working directory in a new folder called work-arch.'
	echo ' - The customized image will be saved as ubuntu-xx.xx-desktop-arch-gnomeshell-remix.iso in the current working directory.'
	exit 64
}

depends() {
	echo 'E: This script needs debootstrap, squashfs-tools, genisoimage and lzma installed.' >&2
	exit 72
}


if [ $# -ne 2 ] && [ $# -ne 3 ]
then
	usage
fi

cmd="$1"
arch="$2"

if [ $arch != 'i386' ] && [ $arch != 'amd64' ]
then
	usage
fi

scriptdir="$( cd "$( dirname "$0" )" && pwd )"


if [ $cmd = 'customize' ]
then

	src="$3"
	if [ $# -ne 3 ] || [ ! -f $src ]
	then
		usage
	fi

	if [ ! -x /usr/sbin/debootstrap ] || [ ! -x /usr/bin/genisoimage ] || [ ! -x /usr/bin/mksquashfs ] || [ ! -x /usr/bin/lzma ]
	then
		depends
	fi

	gid_orig=$(id -g)
	uid_orig=$(id -u)

	# create working directory
	mkdir -p work-$arch/extract-cd
	mkdir -p work-$arch/mnt
	cd work-$arch

	# mount original disk image
	sudo mount -o loop ../$src mnt

	# copy files from disk image
	# the slash after mnt is important, otherwise the files are copied to extract-cd/mnt
	# -r: recursive, -l: links, -t: times, --chmod: make files readable
	rsync --exclude=/casper/* -rlt --chmod=ugo=rwX mnt/ extract-cd

	# create the base image
	sudo debootstrap --arch=$arch quantal edit

	#  set up network and mount devfs for chroot
	sudo cp /etc/hosts edit/etc/
	sudo cp /etc/resolv.conf edit/etc/
	sudo mount --bind /dev/ edit/dev

	# add universe repos to the chroot sources.list
	sudo sed -i 's/main/main\ universe\ restricted/g' edit/etc/apt/sources.list

	# copy the install script to the chroot
	sudo cp $scriptdir/livecd-script-chroot.sh edit/root/livecd.sh
	sudo chmod +x edit/root/livecd.sh

	# execute the script within the chroot
	sudo chroot edit '/root/livecd.sh'

	# Enable autologin for "Try Ubuntu" option
	sudo sed -i 's/#\ \ AutomaticLoginEnable/AutomaticLoginEnable/' edit/etc/gdm/custom.conf
	sudo sed -i 's/#\ \ AutomaticLogin\ =\ user1/AutomaticLogin\ =\ ubuntu/' edit/etc/gdm/custom.conf

	# delete temporary data from the chroot
	sudo rm -rf edit/tmp/* edit/root/.bash_history edit/etc/hosts edit/var/lib/dbus/machine-id edit/root/livecd-script.sh

	# unmount devfs in chroot
	sudo umount edit/dev

	# copy new kernel from edited rootfs
	zcat edit/boot/initrd.img-*-generic | lzma -c -9 > extract-cd/casper/initrd.lz
	sudo cp edit/boot/vmlinuz-*-generic extract-cd/casper/vmlinuz
	sudo chown $gid_orig:$uid_orig extract-cd/casper/vmlinuz

	# recreate squashfs from the extracted and changed rootfs
	sudo mksquashfs edit extract-cd/casper/filesystem.squashfs -wildcards -e boot/vmlinuz-* boot/initrd.img-*

	# create new filesystem.manifest from the installed package list
	sudo chroot edit dpkg-query -W --showformat='${Package} ${Version}\n' > extract-cd/casper/filesystem.manifest

	# copy original filesystem.manifest-remove and change it
	cp mnt/casper/filesystem.manifest-remove extract-cd/casper/filesystem.manifest-remove
	chmod +w extract-cd/casper/filesystem.manifest-remove
#	sed -i '/ubiquity-slideshow-ubuntu/d' extract-cd/casper/filesystem.manifest-remove

	# get size of the files in the rootfs, as squashfs doesn't allow reading the unpacked size
	sudo du -sx --block-size=1 edit | cut -f1 > extract-cd/casper/filesystem.size
	sudo chmod a-x+rw extract-cd/casper/filesystem.squashfs

	# set up metadata
	sed -i 's/Ubuntu/Ubuntu\ GNOME\ Remix/g' extract-cd/autorun.inf extract-cd/boot/grub/*.cfg
	sed -i "1c\#define DISKNAME  Ubuntu GNOME Remix 12.10 \"Quantal Quetzal\" - Alpha $arch" extract-cd/README.diskdefines
	echo 'https://wiki.ubuntu.com/UbuntuGNOME/ReleaseNotes' > extract-cd/.disk/release_notes_url
	echo "Ubuntu GNOME Remix 12.10 \"Quantal Quetzal\" - Alpha $arch($(date +%Y%m%d))" > extract-cd/.disk/info

	# set up custom bootsplash
	cp $scriptdir/bootsplash/*.pcx extract-cd/isolinux/
	sed -i '1c\foreground=0xFFFFFF' extract-cd/isolinux/gfxboot.cfg
	sed -i '2c\background=0x000000' extract-cd/isolinux/gfxboot.cfg
	sed -i '3c\screen-colour=0x4169a4' extract-cd/isolinux/gfxboot.cfg

	cd extract-cd

	# create md5 sums for check disk feature containing the same files as Ubuntu's default md5 list
	find ./casper ./dists ./pics ./.disk ./efi ./autorun.inf ./install ./preseed ./pool ./boot ./wubi.exe ./README.diskdefines -type f -print0 | xargs -0 md5sum > md5sum.txt

	# generate new disk image
	genisoimage -D -r -V 'Ubuntu GNOME Remix 12.10 Alpha' -cache-inodes -J -l -b isolinux/isolinux.bin -c isolinux/boot.cat -no-emul-boot -boot-load-size 4 -boot-info-table -o ../../quantal-gnome-$arch-$(date -u +%Y%m%d).iso .

	# unmount original disk image
	sudo umount ../mnt

	# get back to our original directory
	cd ../..

elif [ $cmd = 'clean' ]
then

	# ensure that everything is unmounted in case something went wrong
	sudo umount -lf work-$arch/edit/proc
	sudo umount -lf work-$arch/edit/sys
	sudo umount -lf work-$arch/edit/dev
	sudo umount -lf work-$arch/edit/dev/pts
	sudo umount -lf work-$arch/mnt

	# delete the work folder
	sudo rm -rf work-$arch

else
	usage
fi
